// conf.cpp
// cRadio (Carl Radio) v0.9, by Carl van Schaik
// e-mail carl@leg.uct.ac.za
//
// Radiotrack II implementation : Leendert Meyer (leendert@usa.net).
//
// RadioTrack II Protocol : Ben Pfaff

#ifndef CONF_CPP
#define CONF_CPP

#include "conf.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

configFile::configFile()
{
  numMore=0;
  basePort=0;
  cardType=NONE;

  userCfg=new char[40];
  strcpy(userCfg,"/~/.cRadio");

  for(int i=0;i<100;i++)
    moreName[i]=NULL;

  verbose=TRUE;
  defaultChannel=0;
  noVolumeSet=0;
  noDefaultSet=0;
  sleepT=0;
  volume=7;
  guiStyle=MotifStyle;

  alarmHr = 0;
  alarmMin = 0;
  alarmSet = 0;

  int i;
  for (i=0;i<9;i++)
    {
      setFreq(i,0);
      channelName[i] = new char[9];
      strcpy(channelName[i],"--");
    }
}

char* doName(char* n)
{
  char* m = new char[strlen(n)];
  strcpy (m,n);
  for (int p=0;p<strlen(m);p++)
    if (m[p] == ' ') m[p] = '_';
  return m;
}

int configFile::save()
{
  FILE *ccf;  //cRadio config file
  ccf = fopen("/etc/cRadio.conf","w");
  if (ccf==NULL) { return 1; }

  fprintf(ccf,"# /etc/cRadio.conf\n");
  fprintf(ccf,"# cRadio configuration file\n");
  fprintf(ccf,"# cRadio by C. van Schaik, 1998\n");
  fprintf(ccf,"#\n");
  fprintf(ccf,"# You should not be editing this file, it should be done\n");
  fprintf(ccf,"# automatically using the Options and Preset configuration.\n");
  fprintf(ccf,"\n");

  fprintf(ccf,"*verbose=");
  if (verbose) fprintf(ccf,"on ");
  else fprintf(ccf,"off");
  fprintf(ccf,"      # Verbose startup messages\n\n");

  fprintf(ccf,"*basePort=");
  fprintf(ccf,"0x%x \n\n",basePort);

  fprintf(ccf,"*cardType=");
  if (cardType == RADIOTRACK)
    fprintf(ccf,"rtrack");
  else if (cardType == ZOLTRIX)
    fprintf(ccf,"zoltrix");
  else if (cardType == RADIOTRACKII)
    fprintf(ccf,"rtrackII");
  else if (cardType == KERNELDEV)
    fprintf(ccf,"kerneldev");
  fprintf(ccf,"  # rtrack : Use the Aimslab radiocard
                  # zoltrix : Use Zoltrix radio plus, kerneldev=/dev/radio\n\n");

  fprintf(ccf,"*volume=");
  fprintf(ccf,"%i\n",volume);

  fprintf(ccf,"*defaultChannel=");
  fprintf(ccf,"%i\n",defaultChannel);

  fprintf(ccf,"*noVolumeSet=");
  if (noVolumeSet) fprintf(ccf,"1");
  else fprintf(ccf,"0");
  fprintf(ccf,"    # 0 sets volume on startup, 1 disables\n");

  fprintf(ccf,"*noDefaultSet=");
  if (noDefaultSet) fprintf(ccf,"1");
  else fprintf(ccf,"0");
  fprintf(ccf,"   # 0 sets channel on startup, 1 disables\n");

  fprintf(ccf,"*GUI=");
  if (guiStyle == MotifStyle)
    fprintf(ccf,"Motif");
  else if (guiStyle == WindowsStyle)
    fprintf(ccf,"Windows");
  fprintf(ccf, "        # Windows or Motif\n\n");

  fprintf(ccf,"*SleepTime=");
  fprintf(ccf,"%i     #Time in minutes\n\n",sleepT);

  fprintf(ccf,"*alarmTime=");
  fprintf(ccf,"%i:%i    # 24hr time ie 9pm = 21:00\n",alarmHr, alarmMin);

  fprintf(ccf,"*alarmOnStart=");
  fprintf(ccf,"%i   # 1 = true, 0 = false\n\n",alarmSet);

  int q;
  fprintf(ccf,"# Use '_' to map to a space ie 5_FM goes to '5 FM'\n");

  for (q=0;q<(9+numMore);q++)
    fprintf(ccf,"*channel  %2i   %3.2f   %s\n",q,(getFreq(q)/20.0+87),doName(getName(q)));

  fclose(ccf);
  return 0;
}

int configFile::readin()
{
  FILE *ccf;  //cRadio config file
  ccf = fopen("/etc/cRadio.conf","r");
  if (ccf==NULL) { return 10; }
  char *temp = new char[80];
  int s,t=0,tem1,tem2=0;

  printf("Reading in /etc/cRadio.conf\n");
  while (t!=2)
    {
      t = 0;
      while (t==0)
	{
	  s = fgetc(ccf);
	  switch (s)
	    {
	    case '#':
	      {
		while ((fgetc(ccf)!='\n') && (!feof(ccf))) {};
		break;
	      }
	    case '*': t = 1; break;
	    }
	  if (feof(ccf)) t = 2;
	}

      if (t==1)
	{
	  fgets (temp,80,ccf);
	  delSlashN(temp);

	  tem1 = processString(temp);
	  if (tem1) tem2 = tem1;
	}
    }

  delete temp;
  fclose(ccf);
  return tem2;
}

int configFile::processString(char* str)
{
  char* token;
  char* delim = new char[15];
  char* delim2 = new char[15];
  char* delim3 = new char[15];
  int i;

  strcpy(delim,"= \t0123456789");
  strcpy(delim2,"= \t");
  strcpy(delim3,"= :hm\t");

  token = strtok(str,delim);

  if (strcasecmp(token,"basePort") == 0)
    {
      sscanf(strtok(NULL,delim2),"%x",&basePort);   /* get port number */
      if (verbose) printf("Using base port 0x%x\n",basePort);
      if (!((basePort == 0x20f) || (basePort == 0x30f) || (basePort == 0x20c) || (basePort == 0x30c)))
	return 20;
    } else
  if (strcasecmp(token,"cardType") == 0)
    {
      token = strtok(NULL,delim);

      if (strcasecmp(token,"rtrack") == 0)
	cardType=RADIOTRACK;
      else
      if (strcasecmp(token,"zoltrix") == 0)
	cardType=ZOLTRIX;
      else
      if (strcasecmp(token,"rtrackII") == 0)
        cardType=RADIOTRACKII;
       else
      if (strcasecmp(token,"kerneldev") == 0)
	cardType=KERNELDEV;
     else
	return 21;
      if (verbose)
	{
	  if (cardType==RADIOTRACK) printf("Using AimsLab Radiotrack\n");
	  else
            if (cardType==RADIOTRACKII) printf("Using AimsLab Radiotrack II\n");
          else
	    if (cardType==ZOLTRIX) printf("Using Zoltrix radio card\n");
	  else
	    if (cardType==KERNELDEV) printf("Using \"/dev/radio\" radio card\n");
	}
    } else
  if (strcasecmp(token,"numMore") == 0)
    {
      if (verbose) printf("more\n");
    } else
  if (strcasecmp(token,"volume") == 0)
    {
      token = strtok(NULL,delim2);
      if (verbose)
	{
	  printf("Volume = ");
	  printf(token);
	}
      int vol;
      vol = atoi(token);
      if ((vol >=0) && (vol<=16))
	volume=vol;
      else return 22;
    } else
  if (strcasecmp(token,"defaultChannel") == 0)
    {
      token = strtok(NULL,delim2);
      int ch;
      ch = atoi(token);
      if ((ch >=0) && (ch<=9))
	defaultChannel = ch;
      else return 23;

    if (verbose)
      {
	printf("Default channel: ");
	printf(token);
	printf("\n");
      }
    } else
  if (strcasecmp(token,"channel") == 0)
    {
      token = strtok(NULL,delim2);
      int ch;
      ch = atoi(token);
      if ((ch < 0) || (ch > (9+numMore)))
	return 24;

      float f;
      sscanf(strtok(NULL,delim2),"%f",&f);   /* get frequency number */

      int temp;
      f += 0.003;
      temp = int((f-87.0) * 20.0);

      if ((temp < 0) || (temp > 440))
	  return 25;

      token = strtok(NULL,delim2);

      if (token==NULL) return 26;

      if (ch>8)
	{
	  if (numMore < (ch-8))
	    {
	      numMore ++;
	      int* t = new int[numMore];
	      for(i=0; i<(numMore-1); i++)
		t[i]=moreFreq[i];
	      //delete moreFreq;
	      moreFreq=t;
	    }	  
	  moreFreq[numMore-1] = temp;
	  moreName[numMore-1] = new char[strlen(token)+1];
	  for (i=0;i < strlen(token);i++)
	    {
	      if (token[i] == '_') token[i] = ' ';
	      if (token[i] == '\n') token[i] = '\0';
	    }
	  strcpy(moreName[numMore-1],token);
	}
      else
	{
	  channelFreq[ch] = temp;
	  channelName[ch] = new char[strlen(token)+1];
	  for (i=0;i < strlen(token);i++)
	    {
	      if (token[i] == '_') token[i] = ' ';
	      if (token[i] == '\n') token[i] = '\0';
	    }
	  strcpy(channelName[ch],token);
	}
      if (verbose)
	{
	  printf("Station %.2fmhz ",f);
	  printf("= %s\n",channelName[ch]);
	}
    } else
  if (strcasecmp(token,"verbose") == 0)
    {
      token = strtok(NULL,delim);
      if (strcasecmp(token,"on")==0)
	verbose=TRUE;
      else
	verbose=FALSE;
    } else
  if (strcasecmp(token,"noVolumeSet") == 0)
    {
      token = strtok(NULL,delim2);
      int temp;
      temp = atoi(token);
      if (temp==0)
	noVolumeSet=0;
      else
	noVolumeSet=1;
    } else
  if (strcasecmp(token,"noDefaultSet") == 0)
    {
      token = strtok(NULL,delim2);
      int temp;
      temp = atoi(token);
      if (temp==0)
	noDefaultSet=0;
      else
	noDefaultSet=1;
    } else
  if (strcasecmp(token,"GUI") == 0)
    {
      token = strtok(NULL,delim);
      if (strcasecmp(token,"Windows")==0)
	guiStyle=WindowsStyle;
      else
	guiStyle=MotifStyle;
      if (verbose)
	{
	  if (guiStyle)
	    printf("Using windows style\n");
	  else printf("Using motif style\n");
	}
    } else
  if (strcasecmp(token,"sleepTime") == 0)
    {
      token = strtok(NULL,delim2);
      int t;
      t = atoi(token);
      if ((t >=1) && (t <= 1440))
	sleepT = t;
      else return 28;

    if (verbose)
      {
	printf("Sleep time: ");
	printf(token);
	printf("min\n");
      }
    }else
  if (strcasecmp(token,"alarmTime") == 0)
    {
      token = strtok(NULL,delim3);
      int t;
      t = atoi(token);
      if ((t >= 0) && (t < 24))
	{
	  alarmHr = t;
	  token = strtok(NULL,delim3);
	  int t;
	  t = atoi(token);
	  if ((t >= 0) && (t < 60))
	    {
	      alarmMin = t;
	    }
	  else return 29;
	}
      else
	return 29;
    } else
  if (strcasecmp(token,"alarmOnStart") == 0)
    {
      token = strtok(NULL,delim2);
      int t;
      t = atoi(token);
      if ((t == 0) || (t == 1))
	alarmSet = t;
    }

  while (strtok(NULL,delim)!=NULL) {};  //empty the buffer
  return 0;
}

int configFile::getFreq(int n)
{
  if ((n <=8) && (n >= 0))
    {
      return channelFreq[n];
    }
  else if ((n-8) <= numMore)
    {
      return moreFreq[n-9];
    }
  else return 0;
}

char* configFile::getName(int n)
{
  if ((n <=8) && (n >= 0))
    {
      return channelName[n];
    }
  else if ((n-8) <= numMore)
    {
      return moreName[n-9];
    }
  else return "--";
}

int configFile::getNumMoreChannels()
{
  return numMore;
}

int configFile::getBasePort()
{
  return basePort;
}

int configFile::getDefault()
{
  return defaultChannel;
}

void configFile::setFreq(int n, int f)
{
  if ((f >= 0) && (f <= 440))
      {
	if ((n < 9) && (n >= 0))
	  {
	    channelFreq[n] = f;
	  }
	else if ((n-8) <= numMore)
	  {
	    moreFreq[n-9] = f;
	  }
      }
}

void configFile::setName(int n, const char* Name)
{
  if ((n < 9) && (n >= 0))
    {
      if (channelName[n] != NULL) delete channelName[n];

      channelName[n] = new char[strlen(Name)+1];
      strcpy(channelName[n],Name);
    }
  else if ((n-8) <= numMore)
    {
      if (moreName[n-9] != NULL) delete moreName[n-9];

      moreName[n-9] = new char[strlen(Name)+1];
      strcpy(moreName[n-9],Name);
    }
}


void configFile::insChannel(int pos)
{
  int tem;
  if (pos < (9+numMore))
    {
      numMore++;
      for (tem=(8+numMore);tem>pos;tem--)
	{
	  setName(tem,getName(tem-1));
	  setFreq(tem,getFreq(tem-1));
	}
      setName(pos,"--");
      setFreq(pos,0);
    }
  else
    if (pos == (9+numMore))
      {
	numMore++;
	setName(pos,"--");
	setFreq(pos,0);
      }
}

void configFile::delChannel(int pos)
{
  int tem, tem2=0;
  if (pos < (9+numMore))
    {
      if (numMore == 0) tem2 = 1;
      if (numMore > 0) numMore--;
      for (tem=pos;tem<(8+numMore);tem++)
	{
	  setName(tem,getName(tem+1));
	  setFreq(tem,getFreq(tem+1));
	}
      if (tem2)
	{
	  setName(8,"--");
	  setFreq(8,0);
	}
    }
  else
    if (pos == (9+numMore))
      {
	numMore--;
      }
}

void configFile::setBasePort(int P)
{
  basePort = P;
}

void configFile::setDefault(int n)
{
  if ((n >= 0) && (n < (9+numMore)))
    defaultChannel = n;
}

void configFile::setVolume(int n)
{
  if ((n>=0) && (n<=16))
    volume=n;
}

int configFile::getVolume()
{
  return volume;
}

bool configFile::getVerbose()
{
  return verbose;
}

void configFile::setVerbose(bool v)
{
  verbose = v;
}

int configFile::noVolSet()
{
  return noVolumeSet;
}

void configFile::setNoVolSet(int n)
{
  noVolumeSet = n;
}

int configFile::noDefSet()
{
  return noDefaultSet;
}

void configFile::setNoDefSet(int n)
{
  noDefaultSet = n;
}

CardType configFile::getCardType()
{
  return cardType;
}

void configFile::setCardType(CardType c)
{
  cardType = c;
}

GUIStyle configFile::gui()
{
  return guiStyle;
}

void configFile::setGui(GUIStyle s)
{
  guiStyle = s;
}

int configFile::getSleepTime()
{
  return sleepT;
}

void configFile::setSleepTime(int t)
{
  if ((t >0) && (t<500))
      {
	sleepT = t;
      }
}

void configFile::delSlashN(char* wrd)
{
   int h,i = strlen(wrd);
   //for (h=0;h<i;h++)
   //     if (wrd[h]='\n') wrd[h]=' ';
}

int configFile::getAlarmHr()
{
  return alarmHr;
}

void configFile::setAlarmTime(int Hr,int Min)
{
  if ((alarmHr >= 0) && (alarmHr < 24) && (alarmMin >=0) && (alarmMin < 60))
    {
      alarmHr = Hr;
      alarmMin = Min;
    }
}

int configFile::getAlarmMin()
{
  return alarmMin;
}

int configFile::setAlarm()
{
  return alarmSet;
}

void configFile::setAlarmDef(int i)
{
  if (i)
    alarmSet = 1;
  else
    alarmSet = 0;
}

#endif
