/*
 * idle.c
 * Copyright (C) 2000 Fabio Checconi <fchecconi@libero.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <errno.h>
#include <gnome.h>
#include <math.h>
#include <unistd.h>
#include "calc.h"
#include "dsp.h"
#include "fft.h"
#include "gate.h"
#include "idle.h"
#include "note.h"
#include "tone.h"
#include "globals.h"

extern	int 		_width;
extern	int 		_height;
extern	line_ends	_line_end[101];
extern	t_config	 _config;
extern	GdkFont		*_font;
extern	GdkPixmap	*_background;
extern	GdkPixmap	*_backing_pixmap;
extern	GdkPixmap	*_display;
extern	GdkPixmap	*_arrow[4];
extern	GtkWidget	*_main_window;

gint		_tag;
int		_first_loop = -1;
int		_frags_per_time;
unsigned char 	*audiobuf = NULL;
unsigned char	 *fftbuf = NULL;
_note_type	n;
_dsp_control 	_ctrl = { "/dev/dsp", NULL, -1, 0, RECORD_MODE, 1, DRIVER_FRAGS,
	512, U8, 1, 32768, 0, 0, 0, 0 };

gint
idle_loop( gpointer data )
{
	GtkWidget *widget = (GtkWidget *) data;
	double f;
	long bread;
	int x, y, w, h;

	if( !_first_loop ) {
		if( dsp_avail() == -1 ) {
			error_msg( "Error reading from audio device\0", CUSTOM, 1 );
			return FALSE;
		}
		if( _ctrl.ibytes == 0 || _ctrl.ifrags == _ctrl.frags ) {
			error_msg( "Timed Out\0", CUSTOM, 1 );
			_first_loop = 1;
			return TRUE;
		}
		if( _ctrl.ifrags < _frags_per_time ) {
			usleep( (_frags_per_time - _ctrl.ifrags) * 0.512 / _ctrl.srate );
			return TRUE;
		}
		bread = 512*_ctrl.ifrags;
		if( dsp_record( audiobuf, bread ) == -1 ) {
			error_msg( "Error reading from audio device\0", CUSTOM, 1 );
			return FALSE;
		}
		noise_gate_U8( audiobuf, bread, _ctrl.srate, _config.noise_gate, 5, 5, 5 );
		if( bread < _config.fft_points ) {
			memmove( fftbuf, fftbuf + bread, _config.fft_points - bread );
			memcpy( fftbuf + _config.fft_points - bread, audiobuf, bread );
		} else {
			memcpy( fftbuf, audiobuf + bread - _config.fft_points, _config.fft_points ); 
		}
	} else {
		if( dsp_record( fftbuf, _config.fft_points ) == -1 ) {
			error_msg( "Error reading from audio device\0", CUSTOM, 1 );
			return FALSE;
		}
		noise_gate_U8( fftbuf, _config.fft_points, _ctrl.srate,_config.noise_gate,5,5,5 );
		_first_loop = 0;
	}
	f = calc_freq( fftbuf, _config.fft_points, _ctrl.srate, _config.mode, BYTE );
	if( f < _config.freq_gate )
		f = 1;

	if( note( f, &n ) != 0 ) {
		error_msg( "Note calc. failed !\0", CUSTOM, 1 );
		return FALSE;
	}
		
	gdk_draw_pixmap( _backing_pixmap, widget->style->fg_gc[GTK_WIDGET_STATE( widget )], _background,
		0, 0, 0, 0, _width, _height );
	if( n.name[0] != ' ' ) {
		w = gdk_string_width( _font, n.name );
		h = gdk_string_height( _font, n.name );
		x = ( _width - w ) / 2;
		y = _height - 10 - h;
		gdk_draw_string( _backing_pixmap, _font, widget->style->white_gc, x, y, n.name );

		x = _width / 2;
		y = _height - 60;
		if( n.cents < 0 ) {
			gdk_draw_pixmap( _backing_pixmap, widget->style->fg_gc[GTK_WIDGET_STATE( widget )], _arrow[2],
				0, 0, x - 120, y, 50, 50 );
		}
		if( n.cents > 0 ) {
			gdk_draw_pixmap( _backing_pixmap, widget->style->fg_gc[GTK_WIDGET_STATE( widget )], _arrow[3],
				0, 0, x + 70, y, 50, 50 );
		}
		gdk_draw_line( _backing_pixmap, widget->style->white_gc, _line_end[n.cents+50].x, _line_end[n.cents+50].y,
			_line_end[n.cents+50].x1, _line_end[n.cents+50].y1 );
	}

	gdk_draw_pixmap( widget->window, widget->style->fg_gc[GTK_WIDGET_STATE( widget )], _backing_pixmap,
		0, 0, 0, 0, _width, _height );

	return TRUE;
}

int
loop_config( void )
{
	GtkWidget *widget;

	if( _config.origin == REC_ORIGIN ) {
		if( _first_loop >= 0 ) {
			gtk_idle_remove( _tag );
			calc_end();
			dsp_close();
			g_warning( "Loop configured while running !\n" );
		}
		_ctrl.srate = _config.srate;
		_ctrl.o_mode = RECORD_MODE;
		if( dsp_open( &_ctrl ) == -1 ) {
			error_msg( "Opening device", STD_ERRNO, errno );
			return -1;
		}

		if( _ctrl.fragsize != 512 ) {
			g_warning( "Cannot set fragsize !!!\n" );
		}
		n.delta = (double)((double)_ctrl.srate /(double)_config.fft_points);
		_frags_per_time = (int)((double)((double) _config.delay / 1000.0 ) * (double) _ctrl.srate / (double)_ctrl.fragsize);
		if ( _frags_per_time < 1 )
			_frags_per_time = 1;

		if( audiobuf )
			free( audiobuf );
		if( fftbuf )
			free( fftbuf );

		audiobuf = (char*) xmalloc( sizeof( char ) * _ctrl.buf_size * _ctrl.frags );
		fftbuf = (char*) xmalloc( sizeof( char ) * _config.fft_points );

		_first_loop = 1;
		_tag = gtk_idle_add( idle_loop, (gpointer) lookup_widget( _main_window, "drawingarea" ) );

		return 0;
	} else {
		if( _first_loop >= 0 ) {
			gtk_idle_remove( _tag );
			widget = lookup_widget( _main_window, "drawingarea" );
			gdk_draw_pixmap( widget->window, widget->style->fg_gc[GTK_WIDGET_STATE( widget )],
				_background, 0, 0, 0, 0, _width, _height );
			calc_end();
			dsp_close();
			_first_loop = -1;
		}
		return 0;
	}
}

