/*
 * main_wondow.c
 * Copyright (C) 2000 Fabio Checconi <fchecconi@libero.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <errno.h>
#include <gnome.h>
#include <math.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "dsp.h"
#include "tone.h"
#include "globals.h"
#include "about_window.h"
#include "main_window.h"
#include "prefs_window.h"
#include "splash_window.h"

extern t_config _config;
extern _dsp_control _ctrl;
extern GtkWidget *_about_window;
extern GtkWidget *_prefs_window;

int _width;
int _height;
line_ends _line_end[101];
GdkFont *_font = NULL;
GdkPixmap *_background = NULL;
GdkPixmap *_backing_pixmap = NULL;
GdkPixmap *_display = NULL;
GdkPixmap *_arrow[4] = { NULL, NULL, NULL, NULL };
GtkWidget *_main_window = NULL;

static GnomeUIInfo file_menu_uiinfo[] = {
	GNOMEUIINFO_MENU_EXIT_ITEM ( on_exit_menu_activate, NULL ),
	GNOMEUIINFO_END
};

static GnomeUIInfo settings_menu_uiinfo[] = {
	GNOMEUIINFO_MENU_PREFERENCES_ITEM ( on_preferences_menu_activate, NULL ),
	GNOMEUIINFO_END
};

static GnomeUIInfo help_menu_uiinfo[] = {
	GNOMEUIINFO_MENU_ABOUT_ITEM ( on_about_menu_activate, NULL ),
	GNOMEUIINFO_END
};

static GnomeUIInfo menubar_uiinfo[] = {
	GNOMEUIINFO_MENU_FILE_TREE ( file_menu_uiinfo ),
	GNOMEUIINFO_MENU_SETTINGS_TREE ( settings_menu_uiinfo ),
	GNOMEUIINFO_MENU_HELP_TREE ( help_menu_uiinfo ),
	GNOMEUIINFO_END
};

gboolean
on_main_window_delete_event( GtkWidget *widget, GdkEvent *event, gpointer user_data )
{
	gtk_main_quit();

	return TRUE;
}

void
on_exit_menu_activate( GtkMenuItem *menuitem, gpointer user_data )
{
	gtk_main_quit();
}

void
on_preferences_menu_activate( GtkMenuItem *menuitem, gpointer user_data )
{
	_prefs_window = create_prefs_window();
	gtk_widget_show( _prefs_window );
}

void
on_about_menu_activate( GtkMenuItem *menuitem, gpointer user_data )
{
	_about_window = create_about_window();
	gtk_widget_show( _about_window );
}

void
on_string_clicked( GtkButton *button, gpointer user_data )
{
	unsigned char *tonebuf;
	unsigned long tonelen;

	_config.string = (int) user_data;
	_config.tone_freq = _string_freq[_config.string];

	if( _config.origin == SQR_ORIGIN || _config.origin == SIN_ORIGIN || _config.origin == TRI_ORIGIN ) {
		_ctrl.o_mode = PLAYBACK_MODE;
		if( dsp_open( &_ctrl ) == -1 ) {
			error_msg ( "Opening device: ", STD_ERRNO, errno );
			return;
		}

		tonelen = _ctrl.srate * 5;
		tonebuf = (char *) xmalloc( sizeof( char ) * tonelen );
		tone( tonebuf, tonelen, _config.tone_freq, _ctrl.srate, 50, _config.tone_type );

		dsp_playback( tonebuf, tonelen );
		dsp_close();
		free( tonebuf );
	}
}

gboolean
on_drawingarea_configure_event( GtkWidget *widget, GdkEventConfigure *event, gpointer user_data )
{
	GdkBitmap *mask;
	GdkColormap *colormap;
	gchar *pathname;
	int  x, y, i;
	double a = (M_PI/2500) * 14;
	const char fname[4][25] = { "guituner/arr01.xpm", "guituner/arr02.xpm",
		"guituner/arr03.xpm", "guituner/arr04.xpm" };
	char *fontname = "-Adobe-Utopia-Regular-R-Normal--19-180-75-75-P-101-iso8859-1";

	_width = widget->allocation.width;
	_height = widget->allocation.height;

	if( _background ) {
		gdk_pixmap_unref ( _background );
		gdk_pixmap_unref ( _backing_pixmap );
	}

	if( !_display ) {
		pathname = gnome_pixmap_file( "guituner/back.xpm" );
		colormap = gtk_widget_get_colormap( widget );
		_display = gdk_pixmap_colormap_create_from_xpm( NULL, colormap, &mask, NULL, pathname );
		g_free( pathname );
		gdk_bitmap_unref( mask );
		for ( i = 0; i < 4; i++ ) {
			pathname = gnome_pixmap_file( fname[i] );
			_arrow[i] = gdk_pixmap_colormap_create_from_xpm( NULL, colormap, &mask, NULL, pathname );
			g_free( pathname );
			gdk_bitmap_unref( mask );
		}
	}
	if( !_font ) {
		_font = gdk_font_load( fontname );
	}

	_background = gdk_pixmap_new( widget->window, _width, _height, -1 );
	_backing_pixmap = gdk_pixmap_new( widget->window, _width, _height, -1 );
	gdk_draw_rectangle( _background, widget->style->black_gc, TRUE, 0, 0, _width, _height );
	x = ( _width -385 ) / 2;
	y = _height / 2 - 100;
	gdk_draw_pixmap( _background, widget->style->fg_gc[GTK_WIDGET_STATE( widget )], _display,
		0, 0, x, y, 380, 200 );

	x = _width / 2;
	y = _height / 2 + 75;

	for( i = 0; i < 101; i++ ) {
		_line_end[i].x1 = x + 149 * sin ( a * ( i - 50 ) );
		_line_end[i].y1 = y - 149 * cos ( a * ( i - 50 ) );
		_line_end[i].x = x + 65 * sin ( a * ( i -50 ) );
		_line_end[i].y = y - 65 * cos ( a * ( i - 50 ) );
	}

	x = _width / 2;
	y = _height - 60;
	gdk_draw_pixmap( _background, widget->style->fg_gc[GTK_WIDGET_STATE( widget )], _arrow[0],
		0, 0, x - 120, y, 50, 50 );
	gdk_draw_pixmap( _background, widget->style->fg_gc[GTK_WIDGET_STATE( widget )], _arrow[1],
		0, 0, x + 70, y, 50, 50 );
	gdk_draw_pixmap( _backing_pixmap, widget->style->fg_gc[GTK_WIDGET_STATE( widget )], _background,
		0, 0, 0, 0, _width, _height );

	return TRUE;
}

gboolean
on_drawingarea_expose_event( GtkWidget *widget, GdkEventExpose *event, gpointer user_data )
{
	gdk_draw_pixmap( widget->window, widget->style->fg_gc[GTK_WIDGET_STATE( widget )], _backing_pixmap,
		event->area.x, event->area.y, event->area.x, event->area.y, event->area.width, event->area.height );

	return FALSE;
}

GtkWidget*
create_main_window( void )
{
	GtkWidget *main_window;
	GtkWidget *main_window_box1;
	GtkWidget *menubar;
	GtkWidget *drawingarea;
	GtkWidget *hbox1;
	GtkWidget *button1;
	GtkWidget *button2;
	GtkWidget *button3;
	GtkWidget *button4;
	GtkWidget *button5;
	GtkWidget *button6;
	GtkWidget *logo;

	main_window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_object_set_data( GTK_OBJECT( main_window ), "main_window", main_window );
	gtk_widget_set_usize( main_window, 400, 270 );
	gtk_container_set_border_width( GTK_CONTAINER( main_window ), 2 );
	gtk_window_set_title( GTK_WINDOW( main_window ), "guiTuner 0.02" );
	gtk_window_set_policy( GTK_WINDOW( main_window ), FALSE, TRUE, FALSE );

	main_window_box1 = gtk_vbox_new( FALSE, 2 );
	gtk_widget_ref( main_window_box1 );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "main_window_box1", main_window_box1,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( main_window_box1 );
	gtk_container_add( GTK_CONTAINER( main_window ), main_window_box1 );

	menubar = gtk_menu_bar_new();
	gtk_widget_ref( menubar );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "menubar", menubar,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( menubar );
	gtk_box_pack_start( GTK_BOX( main_window_box1 ), menubar, FALSE, FALSE, 0 );
	gnome_app_fill_menu( GTK_MENU_SHELL( menubar ), menubar_uiinfo,
		NULL, FALSE, 0 );
	gtk_menu_bar_set_shadow_type( GTK_MENU_BAR( menubar ), GTK_SHADOW_ETCHED_IN );

	gtk_widget_ref( menubar_uiinfo[0].widget );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "file_menu",
		menubar_uiinfo[0].widget,
		(GtkDestroyNotify) gtk_widget_unref );

	gtk_widget_ref( file_menu_uiinfo[0].widget );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "exit_menu",
		file_menu_uiinfo[0].widget,
		(GtkDestroyNotify) gtk_widget_unref );

	gtk_widget_ref( menubar_uiinfo[1].widget );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "settings_menu",
		menubar_uiinfo[1].widget,
		(GtkDestroyNotify) gtk_widget_unref );

	gtk_widget_ref( settings_menu_uiinfo[0].widget );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "preferences_menu",
		settings_menu_uiinfo[0].widget,
		(GtkDestroyNotify) gtk_widget_unref );

	gtk_widget_ref( menubar_uiinfo[3].widget );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "help",
		menubar_uiinfo[2].widget,
		(GtkDestroyNotify) gtk_widget_unref );

	gtk_widget_ref( help_menu_uiinfo[0].widget );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "about_menu",
		help_menu_uiinfo[0].widget,
		(GtkDestroyNotify) gtk_widget_unref );

	drawingarea = gtk_drawing_area_new();
	gtk_widget_ref( drawingarea );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "drawingarea", drawingarea,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( drawingarea );
	gtk_box_pack_start( GTK_BOX( main_window_box1 ), drawingarea, TRUE, TRUE, 0 );
	gtk_widget_set_usize( drawingarea, -2, 200 );

	hbox1 = gtk_hbox_new( FALSE, 0 );
	gtk_widget_ref( hbox1 );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "hbox1", hbox1,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( hbox1 );
	gtk_box_pack_start( GTK_BOX( main_window_box1 ), hbox1, FALSE, FALSE, 0 );

	button1 = gtk_button_new_with_label( "E" );
	gtk_widget_ref( button1 );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "button1", button1,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_set_usize( button1, 24, 24 );
	gtk_widget_show( button1 );
	gtk_box_pack_start( GTK_BOX( hbox1 ), button1, FALSE, FALSE, 0 );

	button2 = gtk_button_new_with_label( "A" );
	gtk_widget_ref( button2 );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "button2", button2,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( button2 );
	gtk_box_pack_start( GTK_BOX( hbox1 ), button2, FALSE, FALSE, 0 );
	gtk_widget_set_usize( button2, 24, 24 );

	button3 = gtk_button_new_with_label( "D" );
	gtk_widget_ref( button3 );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "button3", button3,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show ( button3 );
	gtk_box_pack_start ( GTK_BOX ( hbox1 ), button3, FALSE, FALSE, 0 );
	gtk_widget_set_usize ( button3, 24, 24 );

	button4 = gtk_button_new_with_label( "G" );
	gtk_widget_ref( button4 );
	gtk_object_set_data_full( GTK_OBJECT ( main_window ), "button4", button4,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( button4 );
	gtk_box_pack_start( GTK_BOX( hbox1 ), button4, FALSE, FALSE, 0 );
	gtk_widget_set_usize( button4, 24, 24 );

	button5 = gtk_button_new_with_label( "B" );
	gtk_widget_ref( button5 );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "button5", button5,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( button5 );
	gtk_box_pack_start( GTK_BOX( hbox1 ), button5, FALSE, FALSE, 0 );
	gtk_widget_set_usize( button5, 24, 24 );

	button6 = gtk_button_new_with_label( "E" );
	gtk_widget_ref( button6 );
	gtk_object_set_data_full( GTK_OBJECT ( main_window ), "button6", button6,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( button6 );
	gtk_box_pack_start( GTK_BOX( hbox1 ), button6, FALSE, FALSE, 0 );
	gtk_widget_set_usize( button6, 24, 24 );

	logo = create_pixmap( main_window, "guituner/sign.xpm", FALSE );
	gtk_widget_ref( logo );
	gtk_object_set_data_full( GTK_OBJECT( main_window ), "logo", logo,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( logo );
	gtk_box_pack_start( GTK_BOX( hbox1 ), logo, TRUE, TRUE, 0 );
	gtk_misc_set_alignment( GTK_MISC( logo ), 0.7, 0.5 );

	gtk_signal_connect( GTK_OBJECT( main_window ), "delete_event",
		GTK_SIGNAL_FUNC( on_main_window_delete_event ),
		NULL );
	gtk_signal_connect( GTK_OBJECT( drawingarea ), "expose_event",
		GTK_SIGNAL_FUNC( on_drawingarea_expose_event ),
		NULL );
	gtk_signal_connect( GTK_OBJECT( drawingarea ), "configure_event",
		GTK_SIGNAL_FUNC( on_drawingarea_configure_event ),
		NULL );
	gtk_signal_connect( GTK_OBJECT( button1 ), "clicked",
		GTK_SIGNAL_FUNC( on_string_clicked ),
		(gpointer) 0 );
	gtk_signal_connect( GTK_OBJECT( button2 ), "clicked",
		GTK_SIGNAL_FUNC( on_string_clicked ),
		(gpointer) 1 );
	gtk_signal_connect( GTK_OBJECT( button3 ), "clicked",
		GTK_SIGNAL_FUNC ( on_string_clicked ),
		(gpointer) 2 );
	gtk_signal_connect( GTK_OBJECT( button4 ), "clicked",
		GTK_SIGNAL_FUNC( on_string_clicked ),
		(gpointer) 3 );
	gtk_signal_connect( GTK_OBJECT( button5 ), "clicked",
		GTK_SIGNAL_FUNC( on_string_clicked ),
		(gpointer) 4 );
	gtk_signal_connect( GTK_OBJECT( button6 ), "clicked",
		GTK_SIGNAL_FUNC( on_string_clicked ),
		(gpointer) 5 );

	return main_window;
}
