/*
 * note.c: note recognition stuff
 * Copyright (C) 2000 Fabio Checconi <fchecconi@libero.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>
#include "note.h"

#define	NFACT	1.059463094;

double	ref_freq = 440;

double _string_freq[6] = { 82.40688937, 110.0, 146.8323837, 195.9977173, 246.9416505, 329.6275561 };

char	*notes[] = { "A ", "A#", "B ", "C ", "C#", "D ", "D#", "E ", "F ", "F#", "G ", "G#", "A " };
char *blank = "  ";

int
note( double freq, _note_type *n )
{
	int i, k;
	double d, v, e;

	if( freq <= 8 ) {
		n->name = blank;
		n->cents = -49;
		return 0;
	}

	e = ref_freq;
	if( e > freq ) {
		while( e > freq )
			e /= 2;
	} else {
		while( e < freq / 2 )
			e *= 2;
	}

	d = freq / e;
	v = 12 * log( d ) / log( 2 );
	i = (int) rint( v );
	if( i > 12 || i < 0 ) {
		return -1;
	}

	n->name = notes[i];
	n->cents = ( v - i ) * 100;
	for( k = 0; k < i; k++ )
		e *= NFACT;
	e -= freq;
	v = n->delta / 2;
	if( e <= v && e >= -(v) )
		n -> cents = 0;

	return 0;
}
