/*
 * prefs_window.c
 * Copyright (C) 2000 Fabio Checconi <fchecconi@libero.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include <math.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "globals.h"
#include "idle.h"
#include "prefs_window.h"

extern t_config _config;
GtkWidget *_prefs_window = NULL;

opt_menu_item _rate_sel[6] = {
	{ NULL, 44100, "44100" },
	{ NULL, 32768, "32768" },
	{ NULL, 22050, "22050" },
	{ NULL, 16000, "16000" },
	{ NULL, 11025, "11025" },
	{ NULL, 8000, "8000" }
};

opt_menu_item _points_sel[4] = {
	{ NULL, 65536, "65536" },
	{ NULL, 32768, "32768" },
	{ NULL, 16384, "16384" },
	{ NULL, 8192, "8192" }
};

int history[7] = { 0, 1, 1, 1, 5, 80, 41 };

void
on_pref_exit_clicked( GtkButton *button, gpointer user_data )
{
	GtkWidget *widget, *menu_item;
	int i;
	char *name[2][4] = {
		{ "mic_in", "seq_out", "speak_out", "wav_out" },
		{ "mode1", "mode2", "mode3", "" }
	};

	if( (int) user_data == 0 ) {
		_config.origin = SUSPEND;
		if( loop_config() == -1 )
			return;

		for( i = 0; i < 4; i++ ) {
			widget = lookup_widget( _prefs_window, name[0][i] );
			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON ( 
				&(GTK_RADIO_BUTTON( widget )->check_button.toggle_button ) ) ) == TRUE ) {
				_config.origin = history[0] = i;
				_config.tone_type = i;
			}
		}
		widget = lookup_widget( _prefs_window, "srate_sel" );
		menu_item = gtk_menu_get_active( GTK_MENU( GTK_OPTION_MENU( widget )->menu ) );
		for( i = 0; i < 6; i++ ) {
				if( menu_item == _rate_sel[i].widget ) {
					_config.srate = _rate_sel[i].id;
					history[1] = i;
					break;
				}
		}
		widget = lookup_widget( _prefs_window, "points_sel" );
		menu_item = gtk_menu_get_active( GTK_MENU( GTK_OPTION_MENU( widget )->menu ) );
		for( i = 0; i < 4; i++ ) {
				if( menu_item == _points_sel[i].widget ) {
					_config.fft_points = _points_sel[i].id;
					history[2] = i;
					break;
				}
		}
		for( i = 0; i < 3; i++ ) {
			widget = lookup_widget( _prefs_window, name[1][i] );
			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( 
				&(GTK_RADIO_BUTTON( widget )->check_button.toggle_button ) ) ) == TRUE ) {
				_config.mode = history[3] = i;
			}
		}
		widget = lookup_widget( _prefs_window, "hscale1" );
		_config.noise_gate = history[4] = GTK_HSCALE( widget )->scale.range.adjustment->value;

		widget = lookup_widget( _prefs_window, "spinbutton1" );
		_config.delay = history[5] = GTK_SPIN_BUTTON( widget )->adjustment->value;

		widget = lookup_widget( _prefs_window, "spinbutton2" );
		_config.freq_gate = history[6] = GTK_SPIN_BUTTON( widget )->adjustment->value;

		if( loop_config() == -1 )
			return;
	}
	gtk_widget_destroy( _prefs_window );
}

GtkWidget*
create_prefs_window( void )
{
	GtkWidget *prefs_window;
	GtkWidget *vbox2;
	GtkWidget *hbox11;
	GtkWidget *vbox3;
	GtkWidget *label7;
	GSList *s_orig_group = NULL;
	GtkWidget *mic_in;
	GtkWidget *seq_out;
	GtkWidget *speak_out;
	GtkWidget *wav_out;
	GtkWidget *hbox12;
	GtkWidget *label8;
	GtkWidget *srate_sel;
	GtkWidget *srate_sel_menu;
	GtkWidget *hbox13;
	GtkWidget *label9;
	GtkWidget *points_sel;
	GtkWidget *points_sel_menu;
	GtkWidget *vseparator1;
	GtkWidget *vbox4;
	GtkWidget *label13;
	GSList *calc_m_group = NULL;
	GtkWidget *mode1;
	GtkWidget *mode2;
	GtkWidget *mode3;
	GtkWidget *hbox15;
	GtkWidget *gate_label;
	GtkWidget *hscale1;
	GtkWidget *hbox16;
	GtkWidget *label14;
	GtkObject *spinbutton1_adj;
	GtkWidget *spinbutton1;
	GtkWidget *hbox17;
	GtkWidget *label15;
 	GtkObject *spinbutton2_adj;
	GtkWidget *spinbutton2;
	GtkWidget *pixmap1;
	GtkWidget *hseparator1;
	GtkWidget *hbox10;
	GtkWidget *pref_canc;
	GtkWidget *pref_ok;
	int i;
	
	prefs_window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_object_set_data( GTK_OBJECT( prefs_window ), "prefs_window", prefs_window );
	gtk_window_set_title( GTK_WINDOW( prefs_window ), "Preferences..." );
	gtk_window_set_modal( GTK_WINDOW( prefs_window ), TRUE );
	gtk_window_set_policy( GTK_WINDOW( prefs_window ), TRUE, TRUE, TRUE );

	vbox2 = gtk_vbox_new( FALSE, 0 );
	gtk_widget_ref( vbox2 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "vbox2", vbox2,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( vbox2 );
	gtk_container_add( GTK_CONTAINER( prefs_window ), vbox2 );

	hbox11 = gtk_hbox_new( FALSE, 0 );
	gtk_widget_ref( hbox11 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "hbox11", hbox11,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( hbox11 );
	gtk_box_pack_start( GTK_BOX( vbox2 ), hbox11, TRUE, TRUE, 0 );

	vbox3 = gtk_vbox_new( FALSE, 0 );
	gtk_widget_ref( vbox3 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "vbox3", vbox3,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( vbox3 );
	gtk_box_pack_start( GTK_BOX( hbox11 ), vbox3, TRUE, TRUE, 0 );
	gtk_widget_set_usize( vbox3, 190, -2 );
	gtk_container_set_border_width( GTK_CONTAINER( vbox3 ), 4 );

	label7 = gtk_label_new( "Sound origin:" );
	gtk_widget_ref( label7 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "label7", label7,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( label7 );
	gtk_box_pack_start( GTK_BOX( vbox3 ), label7, FALSE, FALSE, 0 );
	gtk_label_set_justify( GTK_LABEL( label7 ), GTK_JUSTIFY_LEFT );
	gtk_misc_set_alignment( GTK_MISC( label7 ), 0.09, 0.5 );

	mic_in = gtk_radio_button_new_with_label( s_orig_group, "Recording device" );
	s_orig_group = gtk_radio_button_group( GTK_RADIO_BUTTON( mic_in ) );
	gtk_widget_ref( mic_in );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "mic_in", mic_in,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( mic_in );
	gtk_box_pack_start( GTK_BOX( vbox3 ), mic_in, FALSE, FALSE, 0 );
	if( history[0] == 0 )
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( mic_in ), TRUE );

	seq_out = gtk_radio_button_new_with_label( s_orig_group, "Square wave" );
	s_orig_group = gtk_radio_button_group( GTK_RADIO_BUTTON( seq_out ) );
	gtk_widget_ref( seq_out );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "seq_out", seq_out,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( seq_out );
	gtk_box_pack_start( GTK_BOX( vbox3 ), seq_out, FALSE, FALSE, 0 );
	if( history[0] == 1 )
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( seq_out ), TRUE );

	speak_out = gtk_radio_button_new_with_label( s_orig_group, "Sine wave" );
	s_orig_group = gtk_radio_button_group( GTK_RADIO_BUTTON ( speak_out ) );
	gtk_widget_ref( speak_out );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "speak_out", speak_out,
		( GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( speak_out );
	gtk_box_pack_start( GTK_BOX( vbox3 ), speak_out, FALSE, FALSE, 0 );
	if( history[0] == 2 )
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( speak_out ), TRUE );

	wav_out = gtk_radio_button_new_with_label( s_orig_group, "Triangular wave" );
	s_orig_group = gtk_radio_button_group( GTK_RADIO_BUTTON( wav_out ) );
	gtk_widget_ref( wav_out );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "wav_out", wav_out,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( wav_out );
	gtk_box_pack_start( GTK_BOX( vbox3 ), wav_out, FALSE, FALSE, 0 );
	if( history[0] == 3 )
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( wav_out ), TRUE );

	hbox12 = gtk_hbox_new( TRUE, 0 );
	gtk_widget_ref( hbox12 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "hbox12", hbox12,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( hbox12 );
	gtk_box_pack_start( GTK_BOX( vbox3 ), hbox12, TRUE, FALSE, 0 );

	label8 = gtk_label_new( "Sampling rate" );
	gtk_widget_ref( label8 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "label8", label8,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( label8 );
	gtk_box_pack_start( GTK_BOX( hbox12 ), label8, FALSE, FALSE, 0 );

	srate_sel = gtk_option_menu_new();
	gtk_widget_ref( srate_sel );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "srate_sel", srate_sel,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( srate_sel );
	gtk_box_pack_start( GTK_BOX( hbox12 ), srate_sel, FALSE, FALSE, 25 );
	srate_sel_menu = gtk_menu_new();

	for( i = 0; i < 6; i++ ) {
		_rate_sel[i].widget = gtk_menu_item_new_with_label( _rate_sel[i].label );
		gtk_widget_show( _rate_sel[i].widget );
		gtk_menu_append( GTK_MENU( srate_sel_menu ), _rate_sel[i].widget );
	}

	gtk_option_menu_set_menu( GTK_OPTION_MENU( srate_sel ), srate_sel_menu );
	gtk_option_menu_set_history( GTK_OPTION_MENU( srate_sel ), history[1] );

	hbox13 = gtk_hbox_new( TRUE, 0 );
	gtk_widget_ref( hbox13 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "hbox13", hbox13,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( hbox13 );
	gtk_box_pack_start( GTK_BOX( vbox3 ), hbox13, TRUE, FALSE, 0 );

	label9 = gtk_label_new( "FFT Points" );
	gtk_widget_ref( label9 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "label9", label9,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( label9 );
	gtk_box_pack_start( GTK_BOX( hbox13 ), label9, FALSE, FALSE, 0 );

	points_sel = gtk_option_menu_new();
	gtk_widget_ref( points_sel );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "points_sel", points_sel,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( points_sel );
	gtk_box_pack_start( GTK_BOX( hbox13 ), points_sel, FALSE, FALSE, 0 );
	points_sel_menu = gtk_menu_new();

	for( i = 0; i < 4; i++ ) {
		_points_sel[i].widget = gtk_menu_item_new_with_label( _points_sel[i].label );
		gtk_widget_show( _points_sel[i].widget );
		gtk_menu_append( GTK_MENU( points_sel_menu ), _points_sel[i].widget );
	}

	gtk_option_menu_set_menu( GTK_OPTION_MENU( points_sel ), points_sel_menu );
	gtk_option_menu_set_history( GTK_OPTION_MENU( points_sel ), history[2] );

	vseparator1 = gtk_vseparator_new();
	gtk_widget_ref( vseparator1 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "vseparator1", vseparator1,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( vseparator1 );
	gtk_box_pack_start( GTK_BOX( hbox11 ), vseparator1, FALSE, TRUE, 0 );
	gtk_widget_set_usize( vseparator1, 20, -2 );

	vbox4 = gtk_vbox_new( FALSE, 0 );
	gtk_widget_ref( vbox4 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "vbox4", vbox4,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( vbox4 );
	gtk_box_pack_start( GTK_BOX( hbox11 ), vbox4, TRUE, TRUE, 0 );
	gtk_container_set_border_width( GTK_CONTAINER( vbox4 ), 2 );

	label13 = gtk_label_new( "Calculus mode:" );
	gtk_widget_ref( label13 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "label13", label13,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( label13 );
	gtk_box_pack_start( GTK_BOX( vbox4 ), label13, FALSE, FALSE, 0 );
	gtk_label_set_justify( GTK_LABEL( label13 ), GTK_JUSTIFY_LEFT );
	gtk_misc_set_alignment( GTK_MISC( label13 ), 0.0900001, 0.5 );

	mode1 = gtk_radio_button_new_with_label( calc_m_group, "Show max" );
	calc_m_group = gtk_radio_button_group( GTK_RADIO_BUTTON( mode1 ) );
	gtk_widget_ref( mode1 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "mode1", mode1,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( mode1 );
	gtk_box_pack_start( GTK_BOX( vbox4 ), mode1, FALSE, FALSE, 0 );
	if ( history[3] == 0 )
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( mode1 ), TRUE );

	mode2 = gtk_radio_button_new_with_label( calc_m_group, "Harmonic count" );
	calc_m_group = gtk_radio_button_group( GTK_RADIO_BUTTON( mode2 ) );
	gtk_widget_ref( mode2 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "mode2", mode2,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( mode2 );
	gtk_box_pack_start( GTK_BOX( vbox4 ), mode2, FALSE, FALSE, 0 );
	if( history[3] == 1 )
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( mode2 ), TRUE );

	mode3 = gtk_radio_button_new_with_label( calc_m_group, "Harmonic detection" );
	calc_m_group = gtk_radio_button_group( GTK_RADIO_BUTTON( mode3 ) );
	gtk_widget_ref( mode3 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "mode3", mode3,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( mode3 );
	gtk_box_pack_start( GTK_BOX( vbox4 ), mode3, FALSE, FALSE, 0 );
	if( history[3] == 2 )
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( mode3 ), TRUE );

	hbox15 = gtk_hbox_new( FALSE, 20 );
	gtk_widget_ref( hbox15 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "hbox15", hbox15,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( hbox15 );
	gtk_box_pack_start( GTK_BOX( vbox4 ), hbox15, FALSE, FALSE, 0 );

	gate_label = gtk_label_new( "Noise gate" );
	gtk_widget_ref( gate_label );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "gate_label", gate_label,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( gate_label );
	gtk_box_pack_start( GTK_BOX( hbox15 ), gate_label, FALSE, FALSE, 0 );

	hscale1 = gtk_hscale_new( GTK_ADJUSTMENT( gtk_adjustment_new( history[4], 0, 127, 0, 0, 0 ) ) );
	gtk_widget_ref( hscale1 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "hscale1", hscale1,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( hscale1 );
	gtk_box_pack_start( GTK_BOX( hbox15 ), hscale1, FALSE, FALSE, 0 );
	gtk_widget_set_usize( hscale1, 152, -2 );
	gtk_scale_set_digits( GTK_SCALE( hscale1 ), 0 );

	hbox16 = gtk_hbox_new( FALSE, 0 );
	gtk_widget_ref( hbox16 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "hbox16", hbox16,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( hbox16 );
	gtk_box_pack_start( GTK_BOX( vbox4 ), hbox16, FALSE, FALSE, 0 );

	label14 = gtk_label_new( "Update delay (ms)" );
	gtk_widget_ref( label14 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "label14", label14,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( label14 );
	gtk_box_pack_start( GTK_BOX( hbox16 ), label14, FALSE, FALSE, 0 );
	gtk_widget_set_usize( label14, 200, -2 );

	spinbutton1_adj = gtk_adjustment_new( history[5], 12, 500, 1, 10, 10 );
	spinbutton1 = gtk_spin_button_new( GTK_ADJUSTMENT( spinbutton1_adj ), 1, 0 );
	gtk_widget_ref( spinbutton1 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "spinbutton1", spinbutton1,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( spinbutton1 );
	gtk_box_pack_start( GTK_BOX( hbox16 ), spinbutton1, FALSE, FALSE, 0 );
	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( spinbutton1 ), TRUE );
	gtk_spin_button_set_update_policy( GTK_SPIN_BUTTON( spinbutton1 ), GTK_UPDATE_IF_VALID );

	hbox17 = gtk_hbox_new( FALSE, 0 );
	gtk_widget_ref( hbox17 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "hbox17", hbox17,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( hbox17 );
	gtk_box_pack_start( GTK_BOX( vbox4 ), hbox17, FALSE, FALSE, 0 );

	label15 = gtk_label_new( "Minimum frequecy allowed" );
	gtk_widget_ref( label15 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "label15", label15,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show ( label15 );
	gtk_box_pack_start ( GTK_BOX ( hbox17 ), label15, FALSE, FALSE, 0 );
	gtk_widget_set_usize ( label15, 200, -2 );

	spinbutton2_adj = gtk_adjustment_new( history[6], 8, 110, 1, 10, 10 );
	spinbutton2 = gtk_spin_button_new( GTK_ADJUSTMENT( spinbutton2_adj ), 1, 0 );
	gtk_widget_ref( spinbutton2 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "spinbutton2", spinbutton2,
		(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show( spinbutton2 );
	gtk_box_pack_start( GTK_BOX( hbox17 ), spinbutton2, FALSE, FALSE, 0 );
	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( spinbutton2 ), TRUE );
	gtk_spin_button_set_update_policy( GTK_SPIN_BUTTON( spinbutton2 ), GTK_UPDATE_IF_VALID );

	pixmap1 = create_pixmap( prefs_window, "guituner/sign.xpm", FALSE );
	gtk_widget_ref( pixmap1 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "pixmap1", pixmap1,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( pixmap1 );
	gtk_box_pack_start( GTK_BOX( vbox4 ), pixmap1, TRUE, TRUE, 0 );
	gtk_misc_set_alignment( GTK_MISC( pixmap1 ), 0.51, 0.5 );

	hseparator1 = gtk_hseparator_new();
	gtk_widget_ref( hseparator1 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "hseparator1", hseparator1,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( hseparator1 );
	gtk_box_pack_start( GTK_BOX( vbox2 ), hseparator1, FALSE, TRUE, 0 );
	gtk_widget_set_usize( hseparator1, -2, 16 );

	hbox10 = gtk_hbox_new( FALSE, 15 );
	gtk_widget_ref( hbox10 );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "hbox10", hbox10,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( hbox10 );
	gtk_box_pack_start( GTK_BOX( vbox2 ), hbox10, FALSE, FALSE, 0 );
	gtk_container_set_border_width( GTK_CONTAINER( hbox10 ), 7 );

	pref_ok = gnome_stock_button( GNOME_STOCK_BUTTON_OK );
	gtk_widget_ref( pref_ok );
	gtk_object_set_data_full( GTK_OBJECT( prefs_window ), "pref_ok", pref_ok,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( pref_ok );
	gtk_box_pack_end( GTK_BOX( hbox10 ), pref_ok, FALSE, FALSE, 0 );
	gtk_widget_set_usize( pref_ok, 94, -2 );

	pref_canc = gnome_stock_button( GNOME_STOCK_BUTTON_CANCEL );
	gtk_widget_ref( pref_canc );
	gtk_object_set_data_full( GTK_OBJECT ( prefs_window ), "pref_canc", pref_canc,
		(GtkDestroyNotify) gtk_widget_unref );
	gtk_widget_show( pref_canc );
	gtk_box_pack_end( GTK_BOX( hbox10 ), pref_canc, FALSE, FALSE, 0 );
	gtk_widget_set_usize( pref_canc, 94, -2 );

	gtk_signal_connect( GTK_OBJECT( pref_canc ), "clicked",
		GTK_SIGNAL_FUNC( on_pref_exit_clicked ),
		(gpointer) 1 );
	gtk_signal_connect( GTK_OBJECT( pref_ok ), "clicked",
		GTK_SIGNAL_FUNC( on_pref_exit_clicked ),
		(gpointer)  0 );

	return prefs_window;
}