/*
 * tone.c
 * Copyright (C) 2000 Fabio Checconi <fchecconi@libero.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "dsp.h"
#include "tone.h"

void	_tone_square	( void );
void	_tone_sine		( void );
void	_tone_triang	( void );

unsigned char *buffer = NULL;
unsigned char ampl = 0;
unsigned long buflen;

void
_tone_square( void )
{
	int i;
	unsigned char *ptr = buffer;

	for( i = 0; i < buflen / 2; i++ )
		*ptr++ = 128 + ampl;
	for( i = buflen / 2; i < buflen; i++ )
		*ptr++ = 128 - ampl;
}

void
_tone_sine( void )
{
	int i;
	unsigned char *ptr = buffer;

	for( i = 0; i < buflen; i++ )
		*ptr++ = 128 + ampl * sin( 2 * M_PI * i / buflen );
}
void
_tone_triang( void )
{
	int i, f = 4 * ampl / buflen, v = 0;
	unsigned char *ptr = buffer;

	for( i = 0; i < buflen / 4; i++ ) {
		*ptr++ = 128 + v;
		v += f;
	}
	for( i = buflen / 4; i < buflen / 2; i++ ) {
		v -= f;
		*ptr++ = 128 + v;
	}
	if( v < 0 )
		v = 0;
	for( i = buflen / 2; i < 3 * buflen / 4; i++ ) {
		v -= f;
		*ptr++ = 128 - v;
	}
	for( i = buflen / 4; i < buflen / 2; i++ ) {
		v += f;
		*ptr++ = 128 - v;
	}
}

int
tone( unsigned char *buf, unsigned long len, double freq, int rate, int amp, int type )
{
	unsigned char *src, *dst = buf;

	buflen = rate / freq;
	buffer = (char *) malloc( sizeof( char ) * buflen );
	ampl = ( amp > 127 ) ? 127 : amp;

	switch( type ) {
		case SQUARE_WAVE:
				_tone_square();
				break;
		case SINE_WAVE:
				_tone_sine();
				break;
		case TRIANG_WAVE:
				_tone_triang();
				break;
		default:
				return -1;
	}

	while( dst < buf + len ) {
		src = buffer;
		while( src < buffer + buflen && dst < buf + len )
			*dst++ = *src++;
	}

	return 0;
}