/*
 *  libradio, a simple fm tuner control library 
 *  Copyright (C) 1999, 2000  Dan Johnson <dj51d@progworks.net>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */
#include <stdlib.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/videodev.h>
#include "radio.h"

int radio_fd = 0;
struct video_tuner *tuner;
struct video_audio *audio;
float radio_saved_freq = 93.7; /* default station */
__u16 volume = 32000;
__u16 bass = 32000; /* can someone suggest a good default value */
__u16 treble = 32000; /* what is a good default value? */

/* proposed change: int radio_init(char *device) */
int radio_init() {
  radio_fd = open("/dev/radio0", O_RDONLY);
  if(radio_fd == -1)
    return RADIO_DEV_OPEN_FAILURE;
  tuner = malloc(sizeof(struct video_tuner));
  audio = malloc(sizeof(struct video_audio));
  if((audio == NULL) || (tuner == NULL))
    return RADIO_MALLOC_ERROR;
  tuner->tuner = 0;
  return RADIO_OK;
}

void radio_exit()
{
  free(tuner);
  free(audio);
}

int set_freq(float freq)
{
  double div;
  unsigned long lfreq;
  if(radio_fd == 0) 
    return RADIO_DEV_NOT_OPEN;
  ioctl(radio_fd, VIDIOCGTUNER, tuner);
  if((tuner->flags & VIDEO_TUNER_LOW) == 0)
    div = 16;
  else
    div = 16000;
  lfreq = freq * div;
  if((lfreq > tuner->rangehigh) || (lfreq < tuner->rangelow))
    return RADIO_BAD_FREQUENCY;
  ioctl(radio_fd, VIDIOCSFREQ, &lfreq);
  return RADIO_OK;
}

int set_bass(__u16 lbass)
{
  bass = lbass;
  if(radio_fd == 0)
    return RADIO_DEV_NOT_OPEN;
  audio->bass = bass;
  audio->volume = volume;
  audio->treble = treble;
  ioctl(radio_fd, VIDIOCSAUDIO, audio); 
  return RADIO_OK;
}

int set_treble(__u16 ltreble)
{
  treble = ltreble;
  if(radio_fd == 0)
    return RADIO_DEV_NOT_OPEN;
  audio->treble = treble;
  audio->volume = volume;
  audio->bass = bass;
  ioctl(radio_fd, VIDIOCSAUDIO, audio);
  return RADIO_OK;
}

int set_volume(__u16 vol)
{
  volume = vol;
  if(radio_fd == 0)
    return RADIO_DEV_NOT_OPEN;
  audio->volume = volume;
  audio->bass = bass;
  audio->treble = treble;
  ioctl(radio_fd, VIDIOCSAUDIO, audio);
  return RADIO_OK;
}

int mute()
{
  if(radio_fd == 0)
    return RADIO_DEV_NOT_OPEN;
  ioctl(radio_fd, VIDIOCGAUDIO, audio);
  get_freq(&radio_saved_freq);
  volume = audio->volume;
  audio->flags |= VIDEO_AUDIO_MUTE;
  ioctl(radio_fd, VIDIOCSAUDIO, audio);
  return RADIO_OK;
}

int unmute()
{
  if(radio_fd == 0)
    return RADIO_DEV_NOT_OPEN;
  ioctl(radio_fd, VIDIOCGAUDIO, audio);
  audio->flags &= ~VIDEO_AUDIO_MUTE;
  set_freq(radio_saved_freq);
  ioctl(radio_fd, VIDIOCSAUDIO, audio);
  return RADIO_OK;
}

int get_freq(float *freq)
{
  double div;
  unsigned long fq;
  if(radio_fd == 0)
    return RADIO_DEV_NOT_OPEN;
  ioctl(radio_fd, VIDIOCGFREQ, &fq);
  ioctl(radio_fd, VIDIOCGTUNER, tuner);
  if((tuner->flags & VIDEO_TUNER_LOW) == 0)
    div = 16;
  else
    div = 16000;
  *freq = (fq / div);
  return RADIO_OK;
}

int get_bass(__u16 *lbass)
{
  if(radio_fd == 0)
    return RADIO_DEV_NOT_OPEN;
  ioctl(radio_fd, VIDIOCGAUDIO, audio);
  *lbass = audio->bass;
  return RADIO_OK;
}

int get_treble(__u16 *ltreble)
{
  if(radio_fd == 0) 
    return RADIO_DEV_NOT_OPEN;
  ioctl(radio_fd, VIDIOCGAUDIO, audio);
  *ltreble = audio->treble;
  return RADIO_OK;
}

int get_volume(__u16 *vol)
{
  if(radio_fd == 0)
    return RADIO_DEV_NOT_OPEN;
  ioctl(radio_fd, VIDIOCGAUDIO, audio);
  *vol = audio->volume;
  return RADIO_OK;
}

int get_signal(__u16 *str) {
  if(radio_fd == 0)
    return RADIO_DEV_NOT_OPEN;
  ioctl(radio_fd, VIDIOCGTUNER, tuner);
  *str = tuner->signal;
  return RADIO_OK;
}

int get_audio_flags(__u32 *flags) {
  if(radio_fd == 0)
    return RADIO_DEV_NOT_OPEN;
  ioctl(radio_fd, VIDIOCGAUDIO, audio);
  *flags = audio->flags;
  return RADIO_OK;
}

int get_audio_mode(__u16 *mode) {
  if(radio_fd == 0)
    return RADIO_DEV_NOT_OPEN;
  ioctl(radio_fd, VIDIOCGAUDIO, audio);
  *mode = audio->mode;
  return RADIO_OK;
}
