/* serial control routines for pcfm - Linux command line
   interface for Gemtek PCFM radio, all the following code
   as well as that in radiocontrol.c & mixerstuff.c is by
   Chris Radek. The pcfm.c code was based on pcradio.c by
   Chris Radek, but heavily modified by Stew Benedict
   (benedict@netcom.com) see pcfm.c for GNU copyleft statement */

#include <termios.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <fcntl.h>
#include <sys/signal.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/termio.h>

#include "pcfm.h"

#define _POSIX_SOURCE 1 /* POSIX compliant source */

int openradio(void)
{
  int device;
  struct termios newtio;

  device = open(DEVICE, O_RDWR | O_NOCTTY);
  if (device <0) {perror(DEVICE); exit(-1); }
#ifdef DEBUG
  printf("%s opened.\nFile descriptor %d.\n",(char *)DEVICE,device);
#endif

  newtio.c_cflag = B9600 | CS8 | CLOCAL;
  newtio.c_iflag = IGNPAR | IGNBRK;
  newtio.c_oflag = 0;
  newtio.c_lflag = 0;
  newtio.c_cc[VMIN]=1;
  newtio.c_cc[VTIME]=0;
  tcsetattr(device,TCSANOW,&newtio);

  return device;
}

void clearrts(int device) {
  unsigned short arg = TIOCM_RTS;
  ioctl(device,TCFLSH,2);
  if(ioctl(device,TIOCMBIC,&arg)==-1) {
    fprintf(stderr,"%s\n",strerror(errno));
    exit(1);
  }
}

void setrts(int device) {
  unsigned short arg = TIOCM_RTS;
  ioctl(device,TCFLSH,2);
  if(ioctl(device,TIOCMBIS,&arg)==-1) {
    fprintf(stderr,"%s\n",strerror(errno));
    exit(1);
  }
}

void cleardtr(int device) {
  unsigned short arg = TIOCM_DTR;
  ioctl(device,TCFLSH,2);
  if(ioctl(device,TIOCMBIC,&arg)==-1) {
    fprintf(stderr,"%s\n",strerror(errno));
    exit(1);
  }
}

void setdtr(int device) {
  unsigned short arg = TIOCM_DTR;
  ioctl(device,TCFLSH,2);
  if(ioctl(device,TIOCMBIS,&arg)==-1) {
    fprintf(stderr,"%s\n",strerror(errno));
    exit(1);
  }
}

void closeradio(int device) {
#ifdef DEBUG
  printf("Closing port...\n");
#endif
  close(device);
  exit(0);
}
